
import pandas as pd
import os
import unittest
from docx import Document
from eda_report import get_word_report


class TestDocumentProperties(unittest.TestCase):

    def setUp(self):
        self.data = pd.DataFrame(range(50))
        self.doc_filename = 'test-report.docx'

    def test_document_creation(self):
        if os.path.isfile(self.doc_filename):
            os.remove(self.doc_filename)
        get_word_report(self.data, title='Test Report',
                        output_filename=self.doc_filename)
        # Check if the report is actually created and saved
        self.assertTrue(os.path.isfile(self.doc_filename))
        # Check if the title in the document is as stipulated
        self.assertEqual(
            Document(self.doc_filename).paragraphs[0].text,
            'Test Report'
        )

    def tearDown(self):
        if os.path.isfile(self.doc_filename):
            os.remove(self.doc_filename)



from pandas import DataFrame, RangeIndex
from eda_report.exceptions import InputError


def clean_column_names(data):
    """Makes sure columns (variables) have meaningful names.

    :param data: The data whose columns are checked
    :type data: ``pandas.DataFrame``
    """
    # Ensure the data has meaningful column names
    if isinstance(data.columns, RangeIndex):
        data.columns = [f'var_{i+1}' for i in data.columns]

    return data


def validate_input_dtype(data):
    """Ensures that the data is of type ``pandas.DataFrame``. If it isn't,
    attempt to explicitly cast it as a ``DataFrame``.

    :param data: The data to process.
    :type data: An array-like, sequence, iterable or dict
    :raises InputError: Raised if the data cannot be converted to a
        ``DataFrame``, as required.
    :return: The data as a pandas ``DataFrame``.
    :rtype: ``pandas.DataFrame``
    """
    if isinstance(data, DataFrame):
        return clean_column_names(data)
    else:
        try:
            # Cast the data as a dataframe
            data = DataFrame(data).infer_objects()
            return clean_column_names(data)
        except Exception:
            raise InputError(
                f'Expected a pandas.Dataframe object, but got {type(data)}.'
            )



class Error(Exception):
    """The base class for exceptions in this package."""
    pass


class InputError(Error):
    """The Exception raised when a given input object is invalid.
    """

    def __init__(self, message):
        """Initialise an instance of :class:`InputError`.

        :param message: A brief description of the mishap detected.
        :type message: str
        """
        self.message = message



import pandas as pd
import unittest
from eda_report.validate import validate_input_dtype, clean_column_names


class TestDataValidation(unittest.TestCase):

    def setUp(self):
        self.data = pd.DataFrame(range(50))

    def test_data_type_validation(self):
        # Check if a dataframe is returned as a dataframe
        self.assertIsInstance(
            validate_input_dtype(self.data), pd.DataFrame
        )
        # Check if a series returns a dataframe
        self.assertIsInstance(
            validate_input_dtype(self.data.squeeze()), pd.DataFrame
        )
        # Check if a sequence returns a dataframe
        self.assertIsInstance(
            validate_input_dtype(range(50)), pd.DataFrame
        )

    def test_column_cleaning(self):
        self.assertEqual(
            clean_column_names(self.data).columns.to_list(),
            ['var_1']
        )



import subprocess
import pandas as pd
import os
import unittest
from docx import Document


class TestCLI(unittest.TestCase):

    def setUp(self):
        pd.DataFrame(range(50)).to_csv('test-data.csv')

    def test_command_line_interface(self):
        # Run the package
        subprocess.run(
            ['python', '-m', 'eda_report', 'test-data.csv', '-o',
             'cli-test-report.docx', '-t', 'CLI Test']
        )
        self.assertTrue(os.path.isfile('cli-test-report.docx'))
        self.assertTrue(
            Document('cli-test-report.docx').paragraphs[0].text,
            'CLI Test'
        )

    def tearDown(self):
        for file in {'test-data.csv', 'cli-test-report.docx'}:
            if os.path.isfile(file):
                os.remove(file)



import pandas as pd
import unittest
from eda_report.multivariate import MultiVariable


class TestBivariateAnalysis(unittest.TestCase):

    def setUp(self):
        self.variables = MultiVariable(
            pd.DataFrame({'A': range(50),
                          'B': list('abcde') * 10,
                          'C': [0, 1] * 25,
                          'D': [1, 3, 5, 7, 9] * 10})
        )

    def test_variable_pairs(self):
        self.assertEqual(
            self.variables.var_pairs, {('A', 'C'), ('A', 'D'), ('C', 'D')}
        )

    def test_graphs_plotted(self):
        self.assertIn(b'\x89PNG',
                      self.variables.joint_correlation_plot.getvalue())
        self.assertIn(b'\x89PNG', self.variables.joint_scatterplot.getvalue())
        self.assertIn(
            b'\x89PNG',
            self.variables.bivariate_scatterplots[('A', 'C')].getvalue())

    def test_correlation_description(self):
        self.assertEqual(
            'virtually no correlation (0.03)',
            self.variables.corr_type[('A', 'C')]
        )



import matplotlib
from io import BytesIO
from matplotlib.figure import Figure


# Matplotlib configuration
matplotlib.rcParams['figure.dpi'] = 150  # resolution (dots per inch)
matplotlib.rcParams['figure.autolayout'] = True  # tight layout
matplotlib.rcParams['savefig.transparent'] = False   # transparent background
matplotlib.rcParams["savefig.edgecolor"] = 'k'  # black frame
matplotlib.rcParams['font.family'] = 'serif'
matplotlib.rcParams['axes.spines.top'] = False  # remove top spine
matplotlib.rcParams['axes.spines.right'] = False  # remove left spine
matplotlib.use('agg')  # use non-interactive matplotlib back-end

Fig = Figure


def savefig(figure):
    """Saves the contents of a matplotlib figure, in PNG format, as bytes in a
    file-like object.

    :param figure: A matplotlib figure with plotted axes.
    :type figure: ``matplotlib.figure.Figure``
    :return: A file-like object with the figure's contents.
    :rtype: ``io.BytesIo``
    """
    graph = BytesIO()
    figure.savefig(graph)
    return graph



import pandas as pd
import unittest
from eda_report.univariate import Variable


class TestVariableTypes(unittest.TestCase):

    def setUp(self):
        self.numeric_variable = Variable(
            pd.Series(range(50), name='numbers')
        )
        self.categorical_variable = Variable(
            pd.Series(list('abcd') * 12, name='letters')
        )
        self.boolean_variable = Variable(
            pd.Series([True, False, True] * 15, name='bool')
        )

    def test_numeric_variable(self):
        # Check if data type is numeric
        self.assertEqual(self.numeric_variable.var_type, 'numeric')
        # Check summary statistics
        self.assertEqual(
            self.numeric_variable.statistics.index.to_list(),
            ['Number of observations', 'Average', 'Standard Deviation',
             'Minimum', 'Lower Quartile', 'Median', 'Upper Quartile',
             'Maximum', 'Skewness', 'Kurtosis']
        )
        self.assertAlmostEqual(
            self.numeric_variable.statistics['Average'], 24.5
        )

    def test_categorical_variable(self):
        # Check if data type is categorical
        self.assertEqual(self.categorical_variable.var_type, 'categorical')
        # Check summary statistics
        self.assertEqual(
            self.categorical_variable.statistics.index.to_list(),
            ['Number of observations', 'Unique values',
             'Mode (Highest occurring value)']
        )
        self.assertEqual(
            self.categorical_variable.statistics['Unique values'], 4
        )

    def test_boolean_variable(self):
        # Check if data type is categorical
        self.assertEqual(self.boolean_variable.var_type, 'categorical')
        # Check summary statistics
        self.assertEqual(
            self.boolean_variable.statistics.index.to_list(),
            ['Number of observations', 'Unique values',
             'Mode (Highest occurring value)']
        )
        self.assertEqual(
            self.boolean_variable.statistics['Mode (Highest occurring value)'],
            True
        )


class TestVariableProperties(unittest.TestCase):

    def setUp(self):
        some_sample = pd.Series(range(50), name='some variable')
        some_sample.iloc[[0, 10, 20]] = None
        self.variable = Variable(some_sample)

    def test_variable_name(self):
        self.assertEqual(self.variable.name, 'some variable')

    def test_unique_values(self):
        self.assertEqual(self.variable.num_unique, 47)

    def test_missing_values(self):
        self.assertEqual(self.variable.missing, '3 (6.00%)')

    def test_graph_plotted(self):
        self.assertIn(b'\x89PNG', self.variable.graphs.getvalue())



import pandas as pd
import os
import unittest
from eda_report.exceptions import InputError
from eda_report.read_file import df_from_file


class TestFileLoader(unittest.TestCase):

    def setUp(self):
        self.data = pd.DataFrame(range(50), columns=['data'])  # sample data

        # Save the sample data as the files
        self.data.to_excel('data.xlsx', index=False)
        self.data.to_csv('data.csv', index=False)

    def test_csv_file_load(self):
        self.assertTrue(df_from_file('data.csv').equals(self.data))

    def test_excel_file_load(self):
        self.assertTrue(df_from_file('data.xlsx').equals(self.data))

    def test_invalid_file(self):
        self.assertRaises(InputError, df_from_file, 'data.some_extension')

    def tearDown(self):
        for file in {'data.csv', 'data.xlsx'}:
            if os.path.isfile(file):
                os.remove(file)



from eda_report.document import ReportDocument
from eda_report.cli import process_cli_args

__version__ = '0.0.6a'


def get_word_report(data, *, title='Exploratory Data Analysis Report',
                    graph_colour='orangered',
                    output_filename='eda-report.docx'):
    """Analyse the data, and generate a report in .docx format.

    :param data: The data to analyse.
    :type data: :class:`pandas.DataFrame`, :class:`pandas.Series`, an array or
        sequence.
    :param title: The top level heading for the generated report, defaults to
        'Exploratory Data Analysis Report'.
    :type title: str, optional
    :param graph_colour: The colour to apply to the generated graphs. Any
        valid matplotlib colors specifier, defaults to 'orangered'.
    :type graph_colour: str, optional
    :param output_filename: The name to give the generated report file,
        defaults to 'basic-report.docx'.
    :type output_filename: str, optional
    """
    ReportDocument(data, title=title, graph_colour=graph_colour,
                   output_filename=output_filename)


def run_from_cli():
    """Create an EDA report using arguments from the command line.
    """
    args = process_cli_args()
    get_word_report(args.infile, title=args.title, graph_colour=args.colour,
                    output_filename=args.outfile)
